﻿
function ShowMsg(...)
	return print('|cffFCF75E'..L["addon_name"]..'|r:', ...)
end

-- Slash commands ---------------------------------------------

SLASH_MSS1 = "/mss"
SlashCmdList["MSS"] = function(str)
	local input, input2 = strsplit(" ", str)
	if input == "lock" then 	
		OptionCommand_LockCombatFrame(true)
		ShowMsg("Locked")
	elseif input == "unlock" then
		OptionCommand_LockCombatFrame(false)
		ShowMsg("Unlocked")
	elseif input == "show" then
		OptionCommand_EnableMSS(1)
		OptionCommand_EnableTalentBar(1)
	elseif input == "hide" then
		OptionCommand_EnableMSS(0)
		OptionCommand_EnableTalentBar(0)
	elseif input == "symbiosis" then
		OptionCommand_ShowSymbiosis()
	else
		InterfaceOptionsFrame_OpenToCategory(MSS_OptionsPanel)
		ShowMsg("/mss lock")
		ShowMsg("/mss unlock")
		ShowMsg("/mss show")
		ShowMsg("/mss hide")
		ShowMsg("/mss symbiosis")		
	end
end

--Options

OptionPanelTable = {}

-- Factory function to generate a checkbox option frame
function MSS_CreateCheckOption(parent, column, row, text, func)
	local option = CreateFrame("CheckButton", parent:GetName().."_"..column.."_"..row, parent, "MSS_OptionPanel_CheckButton_T")
	local anchor
	if column == "left" then
		anchor = "TOPLEFT"
	else 
		anchor = "TOPRIGHT"
	end
	option:SetPoint(anchor, parent, anchor, 20, row * -40)
	option.text:SetText(text)	
	option:SetScript("OnClick", func)
	return option
end

-- Build general frame
local function InitOptions_GeneralFrame(panel)
	panel.option_enableCombatFrame = MSS_CreateCheckOption(
		panel, "left", 1, L["options_enable_mss_text"], function()
			OptionCommand_EnableMSS(panel.option_enableCombatFrame:GetChecked())			
		end
	)
	
	panel.option_enableTalentBar = MSS_CreateCheckOption(
		panel, "left", 2, L["options_enable_talentbar_text"], function()
			OptionCommand_EnableTalentBar(panel.option_enableTalentBar:GetChecked())			
		end
	)
	
	panel.option_enableMinimapButton = MSS_CreateCheckOption(
		panel, "left", 3, L["options_enable_minimap_text"], function()
			OptionCommand_EnableMinimapButton(panel.option_enableMinimapButton:GetChecked())			
		end
	)
end

-- Build Combat Frame
local function InitOptions_CombatFrame(parent)
	parent.combatFrame = CreateFrame("Frame", "CombatFrame", parent)
	parent.combatFrame.name = L["option_title_combatframe"]
	parent.combatFrame.parent = parent.name
	InterfaceOptions_AddCategory(parent.combatFrame)
	
	parent.combatFrame.option_lockCombatFrame = MSS_CreateCheckOption(
		parent.combatFrame, "left", 1, L["options_lock_text"], function()
			OptionCommand_LockCombatFrame(parent.combatFrame.option_lockCombatFrame:GetChecked())			
		end
	)
	
	parent.combatFrame.option_disableMouse = MSS_CreateCheckOption(parent.combatFrame, "left", 2, L["options_disable_mouse_text"], function()
			OptionCommand_DisableMouse(parent.combatFrame.option_disableMouse:GetChecked())			
		end
	)
	
	parent.combatFrame.option_detachComboFrame = MSS_CreateCheckOption(
		parent.combatFrame, "left", 3, L["options_detach_combo_text"], function()
			OptionCommand_Detach(parent.combatFrame.option_detachComboFrame:GetChecked())			
		end
	)
	
	parent.combatFrame.option_hideOutOfCombat = MSS_CreateCheckOption(
		parent.combatFrame, "left", 4, L["options_hide_out_of_combat_text"], function()
			OptionCommand_HideOutOfCombat(parent.combatFrame.option_hideOutOfCombat:GetChecked())			
		end
	)
	
	parent.combatFrame.option_lockFloatAlert = MSS_CreateCheckOption(
		parent.combatFrame, "left", 5, L["options_lock_float_icon_text"], function()
			OptionCommand_LockFloatAlert(parent.combatFrame.option_lockFloatAlert:GetChecked())			
		end
	)
	local button = CreateFrame("Button", nil, parent.combatFrame.option_lockFloatAlert, "UIPanelButtonTemplate")
	button:SetWidth(80)
	button:SetHeight(20)
	button:SetPoint("LEFT", parent.combatFrame.option_lockFloatAlert, "RIGHT", 100, 0)
	button:SetText(L["reset_float_icon"])
	button:RegisterForClicks("AnyUp")		
	button:SetScript("OnClick", function()
		MSS_FloatAlert_PreditorySwiftness:SetPoint("CENTER", UIParent, "CENTER", -100, -100)
		MSS_FloatAlert_DoC:SetPoint("CENTER", UIParent, "CENTER", 100, -100)
	end
	)
	
	parent.combatFrame.option_switchOnShapeshift = MSS_CreateCheckOption(
		parent.combatFrame, "left", 6, L["options_switch_on_shapeshift_text"], function()
			OptionCommand_SwitchOnShapeshift(parent.combatFrame.option_switchOnShapeshift:GetChecked())			
		end
	)
	
	parent.combatFrame.option_hideCooldownText = MSS_CreateCheckOption(
		parent.combatFrame, "left", 7, L["options_hide_cooldown_text_text"], function()
			OptionCommand_HideCooldownText(parent.combatFrame.option_hideCooldownText:GetChecked())
		end
	)
	
	parent.combatFrame.option_uiScale = CreateFrame("Slider", "MSS_OptionPanel_UISlider", parent.combatFrame, "OptionsSliderTemplate")
	parent.combatFrame.option_uiScale:SetWidth(200)
	parent.combatFrame.option_uiScale:SetHeight(20)
	parent.combatFrame.option_uiScale:SetPoint("TOPRIGHT", parent.combatFrame, "TOPRIGHT", -20, -80)
	parent.combatFrame.option_uiScale:SetMinMaxValues(0.5, 1.5)						
	parent.combatFrame.option_uiScale:SetValueStep(0.1)
	parent.combatFrame.option_uiScale:Enable()
	getglobal(parent.combatFrame.option_uiScale:GetName() .. 'Low'):SetText('0.5'); --Sets the left-side slider text (default is "Low").
	getglobal(parent.combatFrame.option_uiScale:GetName() .. 'High'):SetText('1.5'); --Sets the right-side slider text (default is "High").
	getglobal(parent.combatFrame.option_uiScale:GetName() .. 'Text'):SetText(L["title_ui_scale"]); --Sets the "title" text (top-centre of slider).
	parent.combatFrame.option_uiScale:Show()
	parent.combatFrame.option_uiScale:SetScript("OnValueChanged", function()
			getglobal(parent.combatFrame.option_uiScale:GetName() .. 'Text'):SetText(L["title_ui_scale"].." - "..format("%.1f",parent.combatFrame.option_uiScale:GetValue()))						
			MSS_SavedOptions["ui_scale"] = parent.combatFrame.option_uiScale:GetValue()						
			MSS_Mask:SetScale(parent.combatFrame.option_uiScale:GetValue())	
		end
	)	
end

-- Option Panel 
function OptionsPanel_OnLoad(panel)	
	panel.name = L["addon_name"];		
	panel.okay = function(self) end	
	-- panel.cancel = function (self) end
	panel.refresh = RefreshUI
	InterfaceOptions_AddCategory(panel);		
	InitOptions_GeneralFrame(panel)
	InitOptions_CombatFrame(panel)
end

-- Update UI to match the current options
function RefreshUI()	
	--General
	MSS_OptionsPanel.option_enableCombatFrame:SetChecked(MSS_SavedOptions["enable_mss"])
	MSS_OptionsPanel.option_enableTalentBar:SetChecked(MSS_SavedOptions["enable_talentbar"])	
	MSS_OptionsPanel.option_enableMinimapButton:SetChecked(MSS_SavedOptions["enable_minimap"])
	
	--CombatFrame
	MSS_OptionsPanel.combatFrame.option_lockCombatFrame:SetChecked(MSS_SavedOptions["lock_combatframe"])
	MSS_OptionsPanel.combatFrame.option_disableMouse:SetChecked(MSS_SavedOptions["disable_mouse"])
	MSS_OptionsPanel.combatFrame.option_detachComboFrame:SetChecked(MSS_SavedOptions["detach"])
	MSS_OptionsPanel.combatFrame.option_hideOutOfCombat:SetChecked(MSS_SavedOptions["hide_out_of_combat"])
	MSS_OptionsPanel.combatFrame.option_lockFloatAlert:SetChecked(MSS_SavedOptions["lock_float_alert"])
	MSS_OptionsPanel.combatFrame.option_uiScale:SetValue(MSS_SavedOptions["ui_scale"])
	MSS_OptionsPanel.combatFrame.option_switchOnShapeshift:SetChecked(MSS_SavedOptions["switch_on_shapeshift"])
	MSS_OptionsPanel.combatFrame.option_hideCooldownText:SetChecked(MSS_SavedOptions["hide_cooldown_text"])
	
	MSS_OptionsPanel.talentBar.option_lock:SetChecked(MSS_SavedOptions["talentbar_lock"])
end

-- Load from saved variables
function LoadOptions(self, event, ...) 
	MSS_OptionsVersion = 2
	local name = select(1, ...)
	if event == "ADDON_LOADED" and name == "MasterShapeShifter" then		
		-- Initialize default values
		if not MSS_SavedOptions or MSS_SavedOptions["version"] ~= MSS_OptionsVersion then	
			MSS_SavedOptions = {}
			
			--general
			MSS_SavedOptions["enable_mss"] = true
			MSS_SavedOptions["enable_talentbar"] = true
			MSS_SavedOptions["enable_minimap"] = true
			
			--combatframe
			MSS_SavedOptions["lock_combatframe"] = false
			MSS_SavedOptions["disable_mouse"] = false			
			MSS_SavedOptions["detach"] = false
			MSS_SavedOptions["hide_out_of_combat"] = false
			MSS_SavedOptions["lock_float_alert"] = true
			MSS_SavedOptions["ui_scale"] = 1.0				
			MSS_SavedOptions["switch_on_shapeshift"] = true
			MSS_SavedOptions['hide_cooldown_text'] = false
			
			--internal data
			MSS_SavedOptions["minimap_angle"] = 0
			ShowMsg("No profile found. Set to default")
		end
		MSS_SavedOptions["version"] = MSS_OptionsVersion
		-- Apply options		
		OptionCommand_EnableMSS(MSS_SavedOptions["enable_mss"])						
		OptionCommand_LoadMinimapPosition()
		OptionCommand_EnableMinimapButton(MSS_SavedOptions["enable_minimap"])
		
		if MSS_SavedOptions["enable_talentbar"] then
			local loaded, reason = LoadAddOn("MasterShapeShifter_Talentbar")
			if loaded then				
				OptionCommand_EnableTalentBar(MSS_SavedOptions["enable_talentbar"])
				MSS_TalentBar:ApplyOptions(MSS_SavedOptions)
			else 
				ShowMsg("TalentActionBar couldn't load...")
				ShowMsg("Reason: "..reason)
			end
		end
		
		--combat frame
		OptionCommand_LockCombatFrame(MSS_SavedOptions["lock_combatframe"])	
		OptionCommand_DisableMouse(MSS_SavedOptions["disable_mouse"])		
		OptionCommand_Detach(MSS_SavedOptions["detach"])
		OptionCommand_HideOutOfCombat(MSS_SavedOptions["hide_out_of_combat"])	
		OptionCommand_LockFloatAlert(MSS_SavedOptions["lock_float_alert"])			
		OptionCommand_SetScale(MSS_SavedOptions["ui_scale"])		
		OptionCommand_SwitchOnShapeshift(MSS_SavedOptions["switch_on_shapeshift"])		
		OptionCommand_HideCooldownText(MSS_SavedOptions["hide_cooldown_text"])
				
		
		
		-- Disable for non-druid classes
		local class, classTag = UnitClass("player")
		if classTag ~= "DRUID" then
			OptionCommand_EnableMSS(false)
			OptionCommand_EnableTalentBar(false)
			--MSS_OptionPanel.option_enableCombatFrame:Disable()
			--MSS_OptionPanel.option_enableTalentBar:Disable()	
		end		
	end
end

-- Option commands -------------
function OptionCommand_LockCombatFrame(flag)
	MSS_SavedOptions["lock_combatframe"] = flag	
	
	if flag then -- lock		
		MSS_Mask:RegisterForDrag("")
		MSS_ComboFrame:RegisterForDrag("")		
	else --unlock		
		MSS_Mask:RegisterForDrag("LeftButton")
		if MSS_SavedOptions["detach"] then
			MSS_ComboFrame:RegisterForDrag("LeftButton")		
		end			
	end
end

function OptionCommand_DisableMouse(flag)
	MSS_SavedOptions["disable_mouse"] = flag
	if flag then		
		MSS_Mask:EnableMouse(false)
		MSS:EnableMouse(false)
		MSS_ComboFrame:EnableMouse(false)
	else		
		MSS_Mask:EnableMouse(true)
		MSS:EnableMouse(true)
		MSS_ComboFrame:EnableMouse(true)
	end	
end


function OptionCommand_EnableMSS(flag)
	MSS_SavedOptions["enable_mss"] = flag			
	if flag then
		MSS_Mask:Show()		
	else 
		MSS_Mask:Hide()		
	end	
end



function OptionCommand_Detach(flag)
	
	MSS_SavedOptions["detach"] = flag	
	if flag then		
		MSS.comboFrame:SetMovable(true)
		MSS.comboFrame:ClearAllPoints()		
		MSS.comboFrame:RegisterForDrag("LeftButton")
	else 			
		MSS.comboFrame:ClearAllPoints()
		MSS.comboFrame:SetPoint("BOTTOMRIGHT", MSS, "TOPRIGHT", -20, -15)
		MSS.comboFrame:SetMovable(false)
		MSS.comboFrame:RegisterForDrag("")		
	end
end

function OptionCommand_HideOutOfCombat(flag)
	MSS_SavedOptions["hide_out_of_combat"] = flag	
	if flag then		
		MSS:Hide()
		MSS_Mask:Hide()
	else		
		MSS:Show()
		MSS_Mask:Show()
	end
end

function OptionCommand_LockFloatAlert(flag) 
	MSS_SavedOptions["lock_float_alert"] = flag		
	if flag then
		MSS_FloatAlert_PreditorySwiftness:EnableMouse(false)
		MSS_FloatAlert_DoC:EnableMouse(false)		
	else 
		MSS_FloatAlert_PreditorySwiftness:EnableMouse(true)
		MSS_FloatAlert_DoC:EnableMouse(true)
		MSS_FloatAlert_PreditorySwiftness:RegisterForDrag("LeftButton")
		MSS_FloatAlert_DoC:RegisterForDrag("LeftButton")
	end
end

function OptionCommand_SetScale(value)	
	MSS_SavedOptions["ui_scale"] = value	
	MSS_Mask:SetScale(value)
end

function OptionCommand_HideCooldownText(flag)
	MSS_SavedOptions["hide_cooldown_text"] = flag		
end

function OptionCommand_SwitchOnShapeshift(flag)
	MSS_SavedOptions["switch_on_shapeshift"] = flag
end

function OptionCommand_EnableTalentBar(flag)
	MSS_SavedOptions["enable_talentbar"] = flag	
	if not IsAddOnLoaded("MasterShapeShifter_Talentbar") then
		local loaded, reason = LoadAddOn("MasterShapeShifter_Talentbar")
			if loaded then
				ShowMsg("TalentActionBar Loaded...")									
			else 
				ShowMsg("TalentActionBar couldn't load...")
				ShowMsg("Reason: "..reason)
				return
			end
	end	
	if flag then		
		MSS_TalentBar:Show()
	else		
		MSS_TalentBar:Hide()
	end		
end

function OptionCommand_ShowSymbiosis()
	local loaded, reason = LoadAddOn("MasterShapeShifter_Symbiosis")
	if loaded then		
		if MSS_SymbiosisFrame:IsVisible() then
			MSS_SymbiosisFrame:Hide()
		else
			MSS_SymbiosisFrame:Show()
		end
	else 
		ShowMsg("Symbiosis couldn't load...")
		ShowMsg("Reason: "..reason)
	end		
end

function OptionCommand_LoadMinimapPosition()
	MSS_MiniMapAngle = MSS_SavedOptions["minimap_angle"]		
	MSS_MinimapButton:UpdatePosition()
end

function OptionCommand_EnableMinimapButton(flag)
	MSS_SavedOptions["enable_minimap"] = flag	
	MSS_MinimapToggle(flag)
end



