local frequency = 1 / 20
local tempo = 0

local MSS_MinimapButton = CreateFrame('Button', 'MSS_MinimapButton', Minimap);
local MSS_MinimapTooltip = CreateFrame("GameTooltip", "MSS_MinimapTooltip", nil, "GameTooltipTemplate")
MSS_MinimapTooltip:SetScale(0.7)

function MSS_MinimapLoad()		
	-- minimap
	MSS_MinimapButton:SetWidth(31); MSS_MinimapButton:SetHeight(31)
	MSS_MinimapButton:SetFrameLevel(8)
	MSS_MinimapButton:RegisterForClicks('anyUp')
	MSS_MinimapButton:RegisterForDrag('LeftButton')
	MSS_MinimapButton:SetHighlightTexture('Interface/Minimap/UI-Minimap-ZoomButton-Highlight')

	local overlay = MSS_MinimapButton:CreateTexture("MSS_MinimapButtonTexture", 'OVERLAY')
	overlay:SetWidth(53); overlay:SetHeight(53)
	overlay:SetTexture('Interface/Minimap/MiniMap-TrackingBorder')
	overlay:SetPoint('TOPLEFT')

	local icon = MSS_MinimapButton:CreateTexture("MSS_MinimapButtonIcon", 'BACKGROUND')
	icon:SetWidth(20); icon:SetHeight(20)
	icon:SetTexture("Interface\\ICONS\\Ability_Druid_MasterShapeshifter")
	icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	icon:SetPoint('TOPLEFT', 7, -5)
	MSS_MinimapButton.icon = icon
	
	MSS_MinimapButton:SetScript('OnDragStart', MSS_MinimapButton.OnDragStart)
	MSS_MinimapButton:SetScript('OnDragStop', MSS_MinimapButton.OnDragStop)
	MSS_MinimapButton:SetScript('OnMouseDown', MSS_MinimapButton.OnMouseDown)
	MSS_MinimapButton:SetScript('OnMouseUp', MSS_MinimapButton.OnMouseUp)
	MSS_MinimapButton:SetScript('OnEnter', MSS_MinimapButton.OnEnter)
	MSS_MinimapButton:SetScript('OnLeave', MSS_MinimapButton.OnLeave)
	MSS_MinimapButton:SetScript('OnClick', MSS_MinimapButton.OnClick)	
	
	MSS_Minimap_Update()	
end

function MSS_MinimapToggle(flag)	
	if flag then
		MSS_MinimapButton:Show()
	else
		MSS_MinimapButton:Hide()
	end
end

function MSS_MinimapButton:OnClick(button)
	if button == 'LeftButton' then
		OptionCommand_ShowSymbiosis()
	elseif button == 'RightButton' then
		InterfaceOptionsFrame_OpenToCategory(MSS_OptionsPanel)
	end
end

function MSS_MinimapButton:OnEnter()
	if not self.dragging then
		MSS_MinimapTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT')
		MSS_MinimapButton:OnTooltipShow(MSS_MinimapTooltip)
		MSS_MinimapTooltip:Show()
	end
end

function MSS_MinimapButton:OnTooltipShow(tooltip)	
	tooltip:AddLine( L["addon_name"].." "..L["addon_version"], 1, 1, 0, nil)
	tooltip:AddDoubleLine(L["left_click"], L["left_click_text"], 0,1,0, 1,1,1)
	tooltip:AddDoubleLine(L["right_click"], L["right_click_text"], 1,0,0, 1,1,1)
end

function MSS_MinimapButton:OnLeave()
	MSS_MinimapTooltip:Hide()
end

function MSS_MinimapButton:OnMouseDown()
	self.icon:SetTexCoord(0, 1, 0, 1)
end

function MSS_MinimapButton:OnMouseUp()
	self.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
end

function MSS_MinimapButton:OnDragStart()
	self.dragging = true
	self:LockHighlight()
	self.icon:SetTexCoord(0, 1, 0, 1)
	self:SetScript('OnUpdate', function(self,elapsed)
			MSS_MinimapButton:Update(elapsed)
	end)			
	MSS_MinimapTooltip:Hide()
end

function MSS_MinimapButton:OnDragStop()
	self.dragging = nil
	self:SetScript('OnUpdate', nil)
	self.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	self:UnlockHighlight()
end

MSS_MiniMapAngle = 0

function MSS_MinimapButton:Update(elapsed)
	tempo = tempo + elapsed
	if tempo < frequency then return end
	tempo = 0	
		
	local mx, my = Minimap:GetCenter()
	local px, py = GetCursorPosition()
	local scale = Minimap:GetEffectiveScale()
	px, py = px / scale, py / scale
	MSS_MiniMapAngle = math.deg(math.atan2(py - my, px - mx)) % 360
	if MSS_SavedOptions then		
		MSS_SavedOptions["minimap_angle"] = MSS_MiniMapAngle
	end
	self:UpdatePosition()	
end

function MSS_MinimapButton:UpdatePosition()
	local angle = math.rad(MSS_MiniMapAngle)
	local cos = math.cos(angle)
	local sin = math.sin(angle)
	local minimapShape = GetMinimapShape and GetMinimapShape() or 'ROUND'

	local round = false
	if minimapShape == 'ROUND' then
		round = true
	elseif minimapShape == 'SQUARE' then
		round = false
	elseif minimapShape == 'CORNER-TOPRIGHT' then
		round = not(cos < 0 or sin < 0)
	elseif minimapShape == 'CORNER-TOPLEFT' then
		round = not(cos > 0 or sin < 0)
	elseif minimapShape == 'CORNER-BOTTOMRIGHT' then
		round = not(cos < 0 or sin > 0)
	elseif minimapShape == 'CORNER-BOTTOMLEFT' then
		round = not(cos > 0 or sin > 0)
	elseif minimapShape == 'SIDE-LEFT' then
		round = cos <= 0
	elseif minimapShape == 'SIDE-RIGHT' then
		round = cos >= 0
	elseif minimapShape == 'SIDE-TOP' then
		round = sin <= 0
	elseif minimapShape == 'SIDE-BOTTOM' then
		round = sin >= 0
	elseif minimapShape == 'TRICORNER-TOPRIGHT' then
		round = not(cos < 0 and sin > 0)
	elseif minimapShape == 'TRICORNER-TOPLEFT' then
		round = not(cos > 0 and sin > 0)
	elseif minimapShape == 'TRICORNER-BOTTOMRIGHT' then
		round = not(cos < 0 and sin < 0)
	elseif minimapShape == 'TRICORNER-BOTTOMLEFT' then
		round = not(cos > 0 and sin < 0)
	end

	local x, y
	if round then
		x = cos*80
		y = sin*80
	else
		x = math.max(-82, math.min(110*cos, 84))
		y = math.max(-86, math.min(110*sin, 82))
	end
	self:SetPoint("CENTER", Minimap, "CENTER", x, y)	
end

function MSS_Minimap_Update()
	MSS_MinimapButton:UpdatePosition();
end 

MSS_MinimapLoad()

 